#!/usr/bin/env python3
import RPi.GPIO as GPIO
from time import sleep
from enum import Enum


class Button(Enum):
    """A helper class """
    BUTTON_0 = 0
    BUTTON_1 = 1
    BUTTON_2 = 2
    BUTTON_3 = 3
    BUTTON_4 = 4
    BUTTON_5 = 5
    BUTTON_6 = 6
    BUTTON_7 = 7
    BUTTON_8 = 8
    BUTTON_9 = 9
    BUTTON_A = 10
    BUTTON_B = 11
    BUTTON_C = 12
    BUTTON_D = 13
    BUTTON_DIAMOND = 14
    BUTTON_UNUSED = -1


class KeyGPIO:

    def __init__(self, cb_fct):
        GPIO.setmode(GPIO.BCM)
        self.MATRIX = [[Button.BUTTON_1, Button.BUTTON_2, Button.BUTTON_3, Button.BUTTON_A], [Button.BUTTON_4, Button.BUTTON_5, Button.BUTTON_6, Button.BUTTON_B], [Button.BUTTON_7, Button.BUTTON_8, Button.BUTTON_9, Button.BUTTON_C], [Button.BUTTON_UNUSED, Button.BUTTON_0, Button.BUTTON_DIAMOND, Button.BUTTON_D]]
        self.ROW = [6, 13, 19, 26]
        self.COL = [12, 16, 20, 21]
        self.green_led = 18
        self.red_led = 17
        self.buzz = 22
        self.relay = 27
        self.callback = cb_fct

        GPIO.setup(self.green_led, GPIO.OUT)
        GPIO.setup(self.red_led, GPIO.OUT)
        GPIO.setup(self.buzz, GPIO.OUT)
        GPIO.setup(self.relay, GPIO.OUT)

        self.set_leds(0, 0)

    def set_leds(self, green, red):
        GPIO.output(self.green_led, green)
        GPIO.output(self.red_led, red)


    def open_door(self):
        GPIO.output(self.relay, 1)
        sleep(1)
        GPIO.output(self.relay, 0)

    def beep(self, times=1):
        for i in range(0, times):
            sleep(.1*(times-1))
            GPIO.output(self.buzz, 1)
            sleep(.1)
            GPIO.output(self.buzz, 0)

    def wait_for_keys(self):
        print("Waiting for keys...")
        for j in range(4):
            GPIO.setup(self.COL[j], GPIO.OUT)
            GPIO.output(self.COL[j], 1)
        for i in range(4):
            GPIO.setup(self.ROW[i], GPIO.IN, pull_up_down=GPIO.PUD_UP)

        try:
            while True:
                for j in range(4):
                    GPIO.output(self.COL[j], 0)
                    for i in range(4):
                        if GPIO.input(self.ROW[i]) == 0:
                            self.callback(self.MATRIX[i][j])
                            sleep(0.5)
                            while GPIO.input(self.ROW[i]) == 0:
                                pass
                    GPIO.output(self.COL[j], 1)
        finally:
            print("End")
            GPIO.cleanup()
